#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include <SDL/SDL.h>

#include "frontend_menu.h"
//#include "frontend_splash.h"
#include "frontend_font.h"

#include "config.h"

#include "port.h"
#include "r3000a.h"
#include "plugins.h"
#include "profiler.h"
#include <SDL.h>
#include "config.h"

#define BIT(number) (1<<(number))
enum {
	WIZ_A = SDLK_RETURN,
	WIZ_B = BIT(21),
	WIZ_X = SDLK_BACKSPACE,
	WIZ_Y = BIT(23),
	WIZ_L = SDLK_LEFT,
	WIZ_R = SDLK_RIGHT,
	WIZ_MENU = BIT(9),
	WIZ_SELECT = BIT(8),
	WIZ_LEFT = BIT(16),
	WIZ_RIGHT = BIT(17),
	WIZ_UP = SDLK_UP,
	WIZ_DOWN = SDLK_DOWN,
	WIZ_VOLUP = BIT(10),
	WIZ_VOLDOWN = BIT(11),
	WIZ_HOME = BIT(13)
};

int game_num_avail=0;
static int last_game_selected=0;
char playemu[] = "pcsx4all\0";
char playgame[256] = "builtinn\0";
char frontend_filename[] = "frontend";
#define NUMGAMES 256
char games[NUMGAMES][256] = {0};

int wiz_freq=700;
int wiz_ramtweaks=0;
int wiz_framelimit=0;
int wiz_frameskip=0;
int wiz_interlace=0;
int wiz_sound=0;
int wiz_psxclock=100;
int wiz_bias=0;
int wiz_core=0;
int wiz_gpu=0;
int wiz_bios=0;
int wiz_fixes=0;
int wiz_alt_fps=0;
int wiz_autosave=0;
static int show_debug = 0;

extern bool frameLimit;
extern int skipCount;
extern bool alt_fps;
extern int SOUND_BUFFER_SIZE;
extern SDL_Surface *screen;

static SDL_Rect dpad_rect = {0,120};
static SDL_Rect button_rect = {176,96};

void SaveConfig(char *file);
int LoadConfig(char *file);

void wiz_video_flip(void){
	SDL_Flip(screen);
	}

static void load_bmp_8bpp(unsigned char *in) 
{
	memcpy(screen->pixels,in,320*240);
}

void wiz_timer_delay(unsigned s)
{
	SDL_Delay(s);
}

static void wiz_text(int x, int y, char *text, int color)
{
	unsigned int i,l;
	Uint8 *s = (Uint8*)(screen->pixels);
	
	s=s+x+y*320;

	for (i=0;i<strlen(text);i++) {
		
		for (l=0;l<8;l++) {
			s[l*320+0]=(fontdata8x8[((text[i])*8)+l]&0x80)?color:s[l*320+0];
			s[l*320+1]=(fontdata8x8[((text[i])*8)+l]&0x40)?color:s[l*320+1];
			s[l*320+2]=(fontdata8x8[((text[i])*8)+l]&0x20)?color:s[l*320+2];
			s[l*320+3]=(fontdata8x8[((text[i])*8)+l]&0x10)?color:s[l*320+3];
			s[l*320+4]=(fontdata8x8[((text[i])*8)+l]&0x08)?color:s[l*320+4];
			s[l*320+5]=(fontdata8x8[((text[i])*8)+l]&0x04)?color:s[l*320+5];
			s[l*320+6]=(fontdata8x8[((text[i])*8)+l]&0x02)?color:s[l*320+6];
			s[l*320+7]=(fontdata8x8[((text[i])*8)+l]&0x01)?color:s[l*320+7];
		}
		s+=8;
	} 
}

void wiz_gamelist_text_out(int x, int y, char *eltexto)
{
	char texto[33];
	strncpy(texto,eltexto,32);
	texto[32]=0;
	if (texto[0]!='-')
		wiz_text(x+1,y+1,texto,0);
	wiz_text(x,y,texto,255);
}

void wiz_gamelist_text_out_fmt(int x, int y, char* fmt, ...)
{
	char strOut[128];
	va_list marker;
	
	va_start(marker, fmt);
	vsprintf(strOut, fmt, marker);
	va_end(marker);	

	wiz_gamelist_text_out(x, y, strOut);
}

unsigned int wiz_joystick_read(int n)
{
	extern int master_volume; 
    unsigned int res=0;

	SDL_Event event;
	
	while(SDL_PollEvent(&event)){
		int key = event.key.keysym.sym;
		switch(event.type){	
			case SDL_KEYDOWN:
				res = key;
				switch(key){
					case SDLK_ESCAPE: exit(-1); break;
					}
				break;
			case SDL_KEYUP:
				break;

				}
			}
	return res;
}

void wiz_deinit(void)
{
	SDL_Quit();
}
static void wiz_intro_screen(void) {

}

static void game_list_sort(void)
{
	int i = 0;
	int j = 0;
	char tmp[256];

	for(i = 0; i < game_num_avail; ++i)
	{
		for(j = i + 1; j < game_num_avail; ++j)
		{
			if(strcmp(games[i], games[j]) > 0)
			{
				strcpy(tmp,games[i]);
				strcpy(games[i],games[j]);
				strcpy(games[j],tmp);
			}
		}
	}
}

void game_list_init(void)
{
	DIR *d=opendir("isos");
	if (d)
	{
		struct dirent *actual=readdir(d);
		while(actual)
		{
			if (((strstr(actual->d_name,".bin")!=NULL) ||
				(strstr(actual->d_name,".img")!=NULL) ||
				(strstr(actual->d_name,".mdf")!=NULL) ||
				(strstr(actual->d_name,".iso")!=NULL)) &&
				(strstr(actual->d_name,".cfg")==NULL))
			{
				strcpy(games[game_num_avail],"isos/");
				strcat(games[game_num_avail],actual->d_name);
				game_num_avail++;
			}
			actual=readdir(d);
		}
		closedir(d);
	}
	d=opendir("exec");
	if (d)
	{
		struct dirent *actual=readdir(d);
		while(actual)
		{
			if (strstr(actual->d_name,".exe")!=NULL)
			{
				strcpy(games[game_num_avail],"exec/");
				strcat(games[game_num_avail],actual->d_name);
				game_num_avail++;
			}
			actual=readdir(d);
		}
		closedir(d);
	}
	
	if (game_num_avail)
	{
		game_list_sort();
	}
}

static void game_list_view(int *pos) {

	int i;
	int view_pos;
	int aux_pos=0;
	int screen_y = 45;
	int screen_x = 38;

	/* Draw background image */
	load_bmp_8bpp(menu_bmp);

	/* Check Limits */
	if (*pos<0)
		*pos=game_num_avail-1;
	if (*pos>(game_num_avail-1))
		*pos=0;
					   
	/* Set View Pos */
	if (*pos<10) {
		view_pos=0;
	} else {
		if (*pos>game_num_avail-11) {
			view_pos=game_num_avail-21;
			view_pos=(view_pos<0?0:view_pos);
		} else {
			view_pos=*pos-10;
		}
	}

	/* Show List */
	for (i=0;i<game_num_avail;i++) {
		if (aux_pos>=view_pos && aux_pos<=view_pos+20) {
			wiz_gamelist_text_out( screen_x, screen_y, games[i]);
			if (aux_pos==*pos) {
				wiz_gamelist_text_out( screen_x-10, screen_y,">" );
				wiz_gamelist_text_out( screen_x-13, screen_y-1,"-" );
			}
			screen_y+=8;
		}
		aux_pos++;
	}
}

static int show_options(char *game)
{
	unsigned long ExKey=0;
	int selected_option=0;
	int x_Pos = 41;
	int y_Pos = 58;
	int options_count = 12;
	char text[256];
	char fnameconf[512];
	FILE *f;

	/* Read game configuration */
	
	
	sprintf(fnameconf,"conf/%s.conf",games[last_game_selected]+5);
	
	LoadConfig(fnameconf);
	
	while(1)
	{
		/* Draw background image */
		load_bmp_8bpp(menu_bmp);

		/* Draw the options */
		strncpy (text,games[last_game_selected],33);
		text[32]='\0';
		wiz_gamelist_text_out(x_Pos,y_Pos-10,text);

		
		//wiz_gamelist_text_out_fmt(x_Pos,0, "Game - %s", game);
		

		//	0	Cpu_type
		switch (Config.Cpu){
			case 0: wiz_gamelist_text_out(x_Pos,y_Pos+10,			"Cpu Type:         Recompiler"); 	break;
			case 1: wiz_gamelist_text_out(x_Pos,y_Pos+10,			"Cpu Type:         Interpreter");	break;
			}
			
		//	1	CPU Clock
		wiz_gamelist_text_out_fmt(x_Pos,y_Pos+20,					"CPU Clock:        %i",wiz_psxclock);

		//	2	CPU Bias
		if (BIAS)
			wiz_gamelist_text_out_fmt(x_Pos,y_Pos+30,				"CPU Bias:         %d",BIAS);
		else
			wiz_gamelist_text_out_fmt(x_Pos,y_Pos+30,				"CPU Bias:         Auto");
			
		//	3	CPU Core
		switch(Config.HLE){
			case 0: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+40,		"CPU Core:         %s","BIOS"); break;
			case 1: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+40,		"CPU Core:         %s","HLE"); break;
			}
			
		//	4	Frame-Limit	
		
		switch(frameLimit){	
			case true: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+50,	"Frame-Limit:      On"); break;
			case false: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+50,	"Frame-Limit:      Off"); break;
			}
			
		//	5	Frame-Skip
		
		wiz_gamelist_text_out_fmt(x_Pos,y_Pos+60,					"Frame-Skip:       %i",skipCount);
		
		//	6	FPS algorithm
		
		switch(alt_fps){
			case false:	wiz_gamelist_text_out_fmt(x_Pos,y_Pos+70,	"FPS algorithm:    Normal"); break;
			case true:	wiz_gamelist_text_out_fmt(x_Pos,y_Pos+70,	"FPS algorithm:    Alternative"); break;
			}
		
		
		//	7	Interlace
		switch(wiz_interlace){
			case 0: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+80,		"Interlace:        %s","OFF"); break;
			case 1: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+80,		"Interlace:        %s","Simple"); break;
			case 2: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+80,		"Interlace:        %s","Progressive"); break;
			}
		
		//	8	Sound
		switch(wiz_sound){
			case 0: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+90,		"Sound:            %s","OFF"); break;
			case 1: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+90,		"Sound:            %s","ON (basic)"); break;
			case 2: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+90,		"Sound:            %s","ON (XA)"); break;
			case 3: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+90,		"Sound:            %s","ON (CD-Audio)"); break;
			case 4: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+90,		"Sound:            %s","ON (XA+CD-Audio)"); break;
			}

		//	9	 GPU Options
		switch(wiz_gpu)
		{
			case 0: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+100,		"GPU Type:         %s","Software"); break;
			case 1: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+100,		"GPU Type:         %s","No Light"); break;
			case 2: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+100,		"GPU Type:         %s","No Blend"); break;
			case 3: wiz_gamelist_text_out_fmt(x_Pos,y_Pos+100,		"GPU Type:         %s","No Light+Blend"); break;
		}
		
		//	10	Game Fixes 
		switch(wiz_fixes)
		{
			case 0: wiz_gamelist_text_out(x_Pos,y_Pos+112,"No Game Fixes"); break;
			case 1: wiz_gamelist_text_out(x_Pos,y_Pos+112,"Sio Irq Always Enabled"); break;
			case 2: wiz_gamelist_text_out(x_Pos,y_Pos+112,"Spu Irq Always Enabled"); break;
			case 3: wiz_gamelist_text_out(x_Pos,y_Pos+112,"Parasite Eve 2, Vandal Hearts 1/2 fix"); break;
			case 4: wiz_gamelist_text_out(x_Pos,y_Pos+112,"InuYasha Sengoku Battle fix"); break;
			case 5: wiz_gamelist_text_out(x_Pos,y_Pos+112,"Abbe's Odyssey fix"); break;
		}
		//	11
		
		wiz_gamelist_text_out_fmt(x_Pos,y_Pos+124,					"Sound Buffer Size:  %i",SOUND_BUFFER_SIZE);
		/*
		switch(show_debug){
			case 0:	wiz_gamelist_text_out(x_Pos,y_Pos+120,"Show info:		No"); break;
			case 1:	wiz_gamelist_text_out(x_Pos,y_Pos+120,"Show info:		FPS"); break;
			case 2:	wiz_gamelist_text_out(x_Pos,y_Pos+120,"Show info:		FPS+Debug"); break;
		}
		*/
		wiz_gamelist_text_out(x_Pos,y_Pos+140,"Press to confirm, <- to return\0");

		/* Show currently selected item */
		wiz_gamelist_text_out(x_Pos-16,y_Pos+(selected_option*10)+10," >");

		wiz_video_flip();
		//while(wiz_joystick_read(0)&0xFFFE00) { wiz_timer_delay(150000); }
		ExKey=wiz_joystick_read(0);// { }
		if(ExKey == WIZ_DOWN){
			selected_option++;
			selected_option = selected_option % options_count;
		}
		else if(ExKey == WIZ_UP){
			selected_option--;
			if(selected_option<0)
				selected_option = options_count - 1;
		}
		else if(ExKey == WIZ_R || ExKey == WIZ_L)
		{
			switch(selected_option) {
			case 0:
				// 0 Config.Cpu
				//if ((ExKey == WIZ_R)||(ExKey == WIZ_L)) Config.Cpu=!Config.Cpu;
				break;
			case 1:
				if(ExKey == WIZ_R){
					wiz_psxclock += 10; /* Add 10% */
					if (wiz_psxclock > 200) wiz_psxclock = 200;
						} else {
							wiz_psxclock -= 10; /* Subtract 10% */
							if (wiz_psxclock < 10) wiz_psxclock = 10;
						}
				break;
			case 2:
				if(ExKey == WIZ_R){
					BIAS++;
					if (BIAS>16) BIAS=0;
						} 
				if(ExKey == WIZ_L){
					BIAS--;
					if (BIAS==-1) BIAS=16;
					}
				break;
			//	Config.HLE
			case 3:
				if ((ExKey == WIZ_R)||(ExKey == WIZ_L)) Config.HLE=!Config.HLE;
				break;
			//	frameLimit
			case 4:
				if ((ExKey == WIZ_R)||(ExKey == WIZ_L)) frameLimit=!frameLimit;
				break;	
				
			case 5:
				if(ExKey == WIZ_R){
					skipCount++;
					if (skipCount>11) skipCount=0;
					} else {
						skipCount--;
						if (skipCount<0) skipCount=11;
						}
				break;
			//	6	FPS algorithm
			case 6:
				if ((ExKey == WIZ_R)||(ExKey == WIZ_L)) alt_fps=!alt_fps;
				break;
			//	7	Interlace	
			case 7:
				//linesInterlace_user++; if (linesInterlace_user>2) linesInterlace_user=0;
				if (ExKey == WIZ_R) wiz_interlace++; if (wiz_interlace>2) wiz_interlace=0;
				if (ExKey == WIZ_L) wiz_interlace--; if (wiz_interlace<0) wiz_interlace=2;
				break;
			//	8	Sound	
			case 8:
				if(ExKey == WIZ_R){
					wiz_sound++;
					if (wiz_sound>4) wiz_sound=0;
					} else {
						wiz_sound--;
						if (wiz_sound<0) wiz_sound=4;
					}
				break;
			//	9	 GPU Options
			case 9:
				wiz_gpu++;
				if (wiz_gpu>3) wiz_gpu=0;
				break;
			//	10	Game Fixes 	
            case 10:
				if(ExKey == WIZ_R){
					wiz_fixes++;
					if (wiz_fixes>5) wiz_fixes=0;
					} else {
						wiz_fixes--;
						if (wiz_fixes<0) wiz_fixes=5;
						}
					break;

			//	show_debug
			case 11:
				if(ExKey == WIZ_R){
					SOUND_BUFFER_SIZE += 1024;
					if (SOUND_BUFFER_SIZE > 1024*20) SOUND_BUFFER_SIZE = 1024*20;
					}
				if(ExKey == WIZ_L){
					SOUND_BUFFER_SIZE -= 1024;
					if (SOUND_BUFFER_SIZE < 1024) SOUND_BUFFER_SIZE = 1024;
					}
				break;
				
				}
			}

		if ((ExKey == WIZ_A) || (ExKey == WIZ_B) || (ExKey == WIZ_MENU)) 
		{
			/* Write game configuration */
//			sprintf(fnameconf,"conf/%s.cfg",games[last_game_selected]+5);
			
			PSXCLK=(u32)((double)PSXCLK*(float)(wiz_psxclock/100.0f));
			
			/* GPU Type */
			extern bool light;
			extern bool blend;
			if ((wiz_gpu==1) || (wiz_gpu==3)) light = false;
			if ((wiz_gpu==2) || (wiz_gpu==3)) blend = false;
			
			switch(wiz_fixes){
				case 0: break;
				case 1: Config.Sio=1; break;
				case 2: Config.SpuIrq=1; break;
				case 3: Config.RCntFix=1; break;
				case 4: Config.VSyncWA=1; break;
				case 5: 
					extern bool enableAbbeyHack;
					enableAbbeyHack=true;
					break;
					}	
					
			/* Sound */
			if (wiz_sound==0) {
				extern bool nullspu; 
				nullspu=true;
				}
				
			if ((wiz_sound==2) || (wiz_sound==4)) Config.Xa=0;
			if ((wiz_sound==3) || (wiz_sound==4)) Config.Cdda=0;
				
			if (wiz_interlace==1) {
				extern int linesInterlace_user; 
				linesInterlace_user=1;
				}
			if (wiz_interlace==2){
				extern bool progressInterlace;
				progressInterlace=true;
				}			
				
			extern bool show_fps;
			int debug_info = 0;
			
			switch(show_debug){
				case 0:
					show_fps = false;
					debug_info = 0;
					break;
				case 1:
					show_fps = true;
					debug_info = 0;
					break;
				case 2:
					show_fps = true;
					debug_info = 1;
					break;
			}
			
			SaveConfig(fnameconf);
			
			/* Selected game will be run */
			return 1;
		}
		else if ((ExKey == WIZ_X) || (ExKey == WIZ_Y) || (ExKey == WIZ_SELECT) || (ExKey == WIZ_HOME))
		{
			/* Return To Menu */
			return 0;
		}
	}
	
}

static void wiz_exit(void)
{
	wiz_deinit();
//	chdir("/usr/gp2x"); /*go to menu*/
//  	execl("gp2xmenu", "gp2xmenu", NULL);
}

extern char savename[256];
void select_game(char *emu, char *game)
{

	unsigned long ExKey;

	/* No Selected game */
	strcpy(game,"builtinn");

	/* Clean screen */
	wiz_video_flip();

	/* Wait until user selects a game */
	while(1)
	{
		game_list_view(&last_game_selected);
		wiz_video_flip();

		//if( (wiz_joystick_read(0) == 0)) 
		//	wiz_timer_delay(100000); 
		ExKey=wiz_joystick_read(0);
		//while((ExKey=wiz_joystick_read(0))!=0)
		//{ 
			//if ((ExKey & WIZ_L) && (ExKey & WIZ_R)) { wiz_exit(); }
		//}

		if (ExKey == WIZ_UP) last_game_selected--;
		if (ExKey == WIZ_DOWN) last_game_selected++;
		if (ExKey == WIZ_L) last_game_selected-=21;
		if (ExKey == WIZ_R) last_game_selected+=21;
		//if (ExKey == WIZ_HOME) wiz_exit();

		if ((ExKey == WIZ_A) || (ExKey == WIZ_B))
		{
			/* Select the game */
			strcpy(game,games[last_game_selected]);
			SetIsoFile(game);
			//strncpy(savename,game,strlen(game)-3);
			sprintf(savename,"save/%s.sav",game+5);
			//printf("savename\%s",savename);
			/* Emulation Options */
			if(show_options(game))
			{
				break;
			}
		}
	}
}

#if 0
int main (int argc, char **argv)
#else
int frontend_run(void)
#endif
{
	/* WIZ Initialization */
	//wiz_init(8,22050,16,0);
	
	//SDL_Init(SDL_INIT_VIDEO);
	screen = SDL_SetVideoMode(320, 240, 8, SDL_HWSURFACE);
	
	/* Show intro screen */
	
	LoadConfig("conf/default.conf");
	
	wiz_intro_screen();

	/* Initialize list of available games */
	game_list_init();
	if (game_num_avail==0)
	{
		wiz_video_flip();
		wiz_gamelist_text_out(35, 110, "ERROR: NO AVAILABLE GAMES FOUND");
		wiz_video_flip();
		SDL_Delay(1500);
		wiz_exit();
	}
	
	/* Select Game */
	select_game(playemu,playgame); 


	
//	exit (0);
}


extern int 	KEY_SQUARE,
			KEY_CIRCLE,
			KEY_TRIANGLE,
			KEY_CROSS,
			KEY_LEFT,
			KEY_RIGHT,
			KEY_UP,
			KEY_DOWN,
			KEY_START,
			KEY_SELECT,
			KEY_L1,
			KEY_L2,
			KEY_R1,
			KEY_R2,
			KEY_SAVE,
			KEY_LOAD,
			KEY_SHOWFPS;			

void SaveConfig(char *file){
	FILE *f;
	f = fopen(file, "wb");

	SetValueld("Config.Cpu",Config.Cpu);
	SetValueld("CPUClock",wiz_psxclock);
	SetValueld("BIAS",BIAS);
	SetValueld("Config.HLE",Config.HLE);
	SetValueld("Frame-Limit",frameLimit);
	SetValueld("Frame-Skip",skipCount);
	SetValueld("FPS-algorithm",alt_fps);
	SetValueld("Interlace",wiz_interlace);
	SetValueld("Sound",wiz_sound);
	SetValueld("GPU-Type",wiz_gpu);
	SetValueld("Fix",wiz_fixes);
	SetValueld("SOUND_BUFFER_SIZE",SOUND_BUFFER_SIZE);
	
	SetValueld("KEY_UP",KEY_UP);
	SetValueld("KEY_DOWN",KEY_DOWN);	
	SetValueld("KEY_LEFT",KEY_LEFT);
	SetValueld("KEY_RIGHT",KEY_RIGHT);

	SetValueld("KEY_SQUARE",KEY_SQUARE);
	SetValueld("KEY_CIRCLE",KEY_CIRCLE);
	SetValueld("KEY_TRIANGLE",KEY_TRIANGLE);
	SetValueld("KEY_CROSS",KEY_CROSS);	
	
	SetValueld("KEY_START",KEY_START);
	SetValueld("KEY_SELECT",KEY_SELECT);
	SetValueld("KEY_L1",KEY_L1);
	SetValueld("KEY_L2",KEY_L2);
	SetValueld("KEY_R1",KEY_R1);
	SetValueld("KEY_R2",KEY_R2);
	
	SetValueld("KEY_SAVE",KEY_SAVE);
	SetValueld("KEY_LOAD",KEY_LOAD);
	SetValueld("KEY_SHOWFPS",KEY_SHOWFPS);
	fclose(f);
	}
	
int LoadConfig(char *file){
	//struct stat buf;
	FILE *f;
	int size;
	char *data, *tmp;
	char path[255];
	//size = buf.st_size;

	f = fopen(file, "r");
	if (f == NULL) return -1;

	fseek(f,0,SEEK_END);
	size = ftell(f);
	fseek(f,0,SEEK_SET);
	
	data = (char*)malloc(size);
	if (data == NULL) return size;
	rewind(f);
	fread(data, 1, size, f);
	fclose(f);
	
	GetValueld("Config.Cpu",Config.Cpu);
	GetValueld("CPUClock",wiz_psxclock);
	GetValueld("BIAS",BIAS);
	GetValueld("Config.HLE",Config.HLE);
	GetValueld("Frame-Limit",frameLimit);
	GetValueld("Frame-Skip",skipCount);
	GetValueld("FPS-algorithm",alt_fps);
	GetValueld("Interlace",wiz_interlace);
	GetValueld("Sound",wiz_sound);
	GetValueld("GPU-Type",wiz_gpu);
	GetValueld("Fix",wiz_fixes);
	GetValueld("SOUND_BUFFER_SIZE",SOUND_BUFFER_SIZE);
	
	GetValueld("KEY_UP",KEY_UP);
	GetValueld("KEY_DOWN",KEY_DOWN);	
	GetValueld("KEY_LEFT",KEY_LEFT);
	GetValueld("KEY_RIGHT",KEY_RIGHT);

	GetValueld("KEY_SQUARE",KEY_SQUARE);
	GetValueld("KEY_CIRCLE",KEY_CIRCLE);
	GetValueld("KEY_TRIANGLE",KEY_TRIANGLE);
	GetValueld("KEY_CROSS",KEY_CROSS);	
	
	GetValueld("KEY_START",KEY_START);
	GetValueld("KEY_SELECT",KEY_SELECT);
	GetValueld("KEY_L1",KEY_L1);
	GetValueld("KEY_L2",KEY_L2);
	GetValueld("KEY_R1",KEY_R1);
	GetValueld("KEY_R2",KEY_R2);
	
	GetValueld("KEY_SAVE",KEY_SAVE);
	GetValueld("KEY_LOAD",KEY_LOAD);
	GetValueld("KEY_SHOWFPS",KEY_SHOWFPS);
	
	free(data);
	
	return 1;
	}